@@ -19,15 +19,17 @@ from __future__ import with_statement
 
 import logging
 import itertools
-import UserDict
 
 from gensim import utils
+from gensim._six import iteritems, iterkeys, itervalues, string_types
+from gensim._six.moves import xrange
+from gensim._six.moves import zip as izip
 
 
 logger = logging.getLogger('gensim.corpora.dictionary')
 
 
-class Dictionary(utils.SaveLoad, UserDict.DictMixin):
+class Dictionary(utils.SaveLoad, dict):
     """
     Dictionary encapsulates the mapping between normalized words and their integer ids.
 
@@ -51,7 +53,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         if len(self.id2token) != len(self.token2id):
             # the word->id mapping has changed (presumably via add_documents);
             # recompute id->word accordingly
-            self.id2token = dict((v, k) for k, v in self.token2id.iteritems())
+            self.id2token = dict((v, k) for k, v in iteritems(self.token2id))
         return self.id2token[tokenid] # will throw for non-existent ids
 
 
@@ -68,7 +70,8 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
 
 
     def __str__(self):
-        return ("Dictionary(%i unique tokens)" % len(self))
+        some_keys = list(itertools.islice(iterkeys(self.token2id), 5))
+        return "Dictionary(%i unique tokens: %s%s)" % (len(self), some_keys, '...' if len(self) > 5 else '')
 
 
     @staticmethod
@@ -84,7 +87,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         This is only a convenience wrapper for calling `doc2bow` on each document
         with `allow_update=True`.
 
-        >>> print Dictionary(["máma mele maso".split(), "ema má máma".split()])
+        >>> print(Dictionary(["máma mele maso".split(), "ema má máma".split()]))
         Dictionary(5 unique tokens)
         """
         for docno, document in enumerate(documents):
@@ -112,7 +115,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         """
         result = {}
         missing = {}
-        if isinstance(document, basestring):
+        if isinstance(document, string_types):
             raise TypeError("doc2bow expects an array of utf8 tokens on input, not a string")
         document = sorted(utils.to_utf8(token) for token in document)
         # construct (word, frequency) mapping. in python3 this is done simply
@@ -137,11 +140,11 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
             self.num_pos += len(document)
             self.num_nnz += len(result)
             # increase document count for each unique token that appeared in the document
-            for tokenid in result.iterkeys():
+            for tokenid in iterkeys(result):
                 self.dfs[tokenid] = self.dfs.get(tokenid, 0) + 1
 
         # return tokenids, in ascending id order
-        result = sorted(result.iteritems())
+        result = sorted(iteritems(result))
         if return_missing:
             return result, missing
         else:
@@ -166,7 +169,8 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         no_above_abs = int(no_above * self.num_docs) # convert fractional threshold to absolute threshold
 
         # determine which tokens to keep
-        good_ids = (v for v in self.token2id.itervalues() if no_below <= self.dfs[v] <= no_above_abs)
+        good_ids = (v for v in itervalues(self.token2id)
+                      if no_below <= self.dfs[v] <= no_above_abs)
         good_ids = sorted(good_ids, key=self.dfs.get, reverse=True)
         if keep_n is not None:
             good_ids = good_ids[:keep_n]
@@ -188,12 +192,20 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         """
         if bad_ids is not None:
             bad_ids = set(bad_ids)
-            self.token2id = dict((token, tokenid) for token, tokenid in self.token2id.iteritems() if tokenid not in bad_ids)
-            self.dfs = dict((tokenid, freq) for tokenid, freq in self.dfs.iteritems() if tokenid not in bad_ids)
+            self.token2id = dict((token, tokenid)
+                                 for token, tokenid in iteritems(self.token2id)
+                                 if tokenid not in bad_ids)
+            self.dfs = dict((tokenid, freq)
+                            for tokenid, freq in iteritems(self.dfs)
+                            if tokenid not in bad_ids)
         if good_ids is not None:
             good_ids = set(good_ids)
-            self.token2id = dict((token, tokenid) for token, tokenid in self.token2id.iteritems() if tokenid in good_ids)
-            self.dfs = dict((tokenid, freq) for tokenid, freq in self.dfs.iteritems() if tokenid in good_ids)
+            self.token2id = dict((token, tokenid)
+                                 for token, tokenid in iteritems(self.token2id)
+                                 if tokenid in good_ids)
+            self.dfs = dict((tokenid, freq)
+                            for tokenid, freq in iteritems(self.dfs)
+                            if tokenid in good_ids)
 
 
     def compactify(self):
@@ -207,12 +219,15 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         logger.debug("rebuilding dictionary, shrinking gaps")
 
         # build mapping from old id -> new id
-        idmap = dict(itertools.izip(self.token2id.itervalues(), xrange(len(self.token2id))))
+        idmap = dict(izip(itervalues(self.token2id),
+                     xrange(len(self.token2id))))
 
         # reassign mappings to new ids
-        self.token2id = dict((token, idmap[tokenid]) for token, tokenid in self.token2id.iteritems())
+        self.token2id = dict((token, idmap[tokenid])
+                             for token, tokenid in iteritems(self.token2id))
         self.id2token = {}
-        self.dfs = dict((idmap[tokenid], freq) for tokenid, freq in self.dfs.iteritems())
+        self.dfs = dict((idmap[tokenid], freq)
+                        for tokenid, freq in iteritems(self.dfs))
 
 
     def save_as_text(self, fname):
@@ -224,7 +239,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
         """
         logger.info("saving dictionary mapping to %s" % fname)
         with utils.smart_open(fname, 'wb') as fout:
-            for token, tokenid in sorted(self.token2id.iteritems()):
+            for token, tokenid in sorted(iteritems(self.token2id)):
                 fout.write("%i\t%s\t%i\n" % (tokenid, token, self.dfs.get(tokenid, 0)))
 
 
@@ -250,7 +265,7 @@ class Dictionary(utils.SaveLoad, UserDict.DictMixin):
 
         """
         old2new = {}
-        for other_id, other_token in other.iteritems():
+        for other_id, other_token in iteritems(other):
             if other_token in self.token2id:
                 new_id = self.token2id[other_token]
             else:
