@@ -6,13 +6,18 @@
 
 
 """
-This module implements the concept of HashDictionary -- a mapping between words
-and their integer ids. The ids are computed as `hash(word) % id_range`, the idea
-being that new words can be represented immediately, without an extra pass through
-the corpus to collect all the ids first. See http://en.wikipedia.org/wiki/Hashing-Trick .
+This module implements the `"hashing trick" <http://en.wikipedia.org/wiki/Hashing-Trick>`_ --
+a mapping between words and their integer ids using a fixed, static mapping. The
+static mapping has a constant memory footprint, regardless of the number of word-types (features)
+in your corpus, so it's suitable for processing extremely large corpora.
 
-This means that, unline plain Dictionary, several words may map to the same id
-(~hash collisions). The word<->id mapping is no longer a bijection.
+The ids are computed as `hash(word) % id_range`, where `hash` is a user-configurable
+function (adler32 by default). Using HashDictionary, new words can be represented immediately,
+without an extra pass through the corpus to collect all the ids first. This is another
+advantage: HashDictionary can be used with non-repeatable (once-only) streams of documents.
+
+A disadvantage of HashDictionary is that, unline plain :class:`Dictionary`, several words may map
+to the same id, causing hash collisions. The word<->id mapping is no longer a bijection.
 
 """
 
@@ -20,17 +25,17 @@ from __future__ import with_statement
 
 import logging
 import itertools
-import UserDict
 import zlib
 
 from gensim import utils
+from gensim._six import iteritems, iterkeys
 
 
 logger = logging.getLogger('gensim.corpora.hashdictionary')
 
 
 
-class HashDictionary(utils.SaveLoad, UserDict.DictMixin):
+class HashDictionary(utils.SaveLoad, dict):
     """
     HashDictionary encapsulates the mapping between normalized words and their
     integer ids.
@@ -158,11 +163,11 @@ class HashDictionary(utils.SaveLoad, UserDict.DictMixin):
             if self.debug:
                 # increment document count for each unique tokenid that appeared in the document
                 # done here, because several words may map to the same tokenid
-                for tokenid in result.iterkeys():
+                for tokenid in iterkeys(result):
                     self.dfs[tokenid] = self.dfs.get(tokenid, 0) + 1
 
         # return tokenids, in ascending id order
-        result = sorted(result.iteritems())
+        result = sorted(iteritems(result))
         if return_missing:
             return result, missing
         else:
@@ -185,13 +190,22 @@ class HashDictionary(utils.SaveLoad, UserDict.DictMixin):
         footprint.
         """
         no_above_abs = int(no_above * self.num_docs) # convert fractional threshold to absolute threshold
-        ok = [item for item in self.dfs_debug.iteritems() if no_below <= item[1] <= no_above_abs]
+        ok = [item for item in iteritems(self.dfs_debug)
+                   if no_below <= item[1] <= no_above_abs]
         ok = frozenset(word for word, freq in sorted(ok, key=lambda item: -item[1])[:keep_n])
 
-        self.dfs_debug = dict((word, freq) for word, freq in self.dfs_debug.iteritems() if word in ok)
-        self.token2id = dict((token, tokenid) for token, tokenid in self.token2id.iteritems() if token in self.dfs_debug)
-        self.id2token = dict((tokenid, set(token for token in tokens if token in self.dfs_debug)) for tokenid, tokens in self.id2token.iteritems())
-        self.dfs = dict((tokenid, freq) for tokenid, freq in self.dfs.iteritems() if self.id2token.get(tokenid, set()))
+        self.dfs_debug = dict((word, freq)
+                              for word, freq in iteritems(self.dfs_debug)
+                              if word in ok)
+        self.token2id = dict((token, tokenid)
+                             for token, tokenid in iteritems(self.token2id)
+                             if token in self.dfs_debug)
+        self.id2token = dict((tokenid, set(token for token in tokens
+                                                 if token in self.dfs_debug))
+                             for tokenid, tokens in iteritems(self.id2token))
+        self.dfs = dict((tokenid, freq)
+                        for tokenid, freq in iteritems(self.dfs)
+                        if self.id2token.get(tokenid, set()))
 
         # for word->document frequency
         logger.info("kept statistics for which were in no less than %i and no more than %i (=%.1f%%) documents" %
