@@ -13,13 +13,16 @@ from __future__ import with_statement
 
 import logging
 import math
-import os
-import itertools
+
+from gensim import utils
 
 import numpy
 import scipy.sparse
 import scipy.linalg
-from scipy.linalg.lapack import get_lapack_funcs, find_best_lapack_type
+from scipy.linalg.lapack import get_lapack_funcs
+
+from gensim._six import iteritems, itervalues, string_types
+from gensim._six.moves import xrange, zip as izip
 
 # scipy is not a stable package yet, locations change, so try to work
 # around differences (currently only concerns location of 'triu' in scipy 0.7 vs. 0.8)
@@ -82,7 +85,7 @@ def corpus2csc(corpus, num_terms=None, dtype=numpy.float64, num_docs=None, num_n
             num_docs = corpus.num_docs
         if num_nnz is None:
             num_nnz = corpus.num_nnz
-    except AttributeError, e:
+    except AttributeError:
         pass # not a MmCorpus...
     if printprogress:
         logger.info("creating sparse matrix from corpus")
@@ -137,8 +140,8 @@ def pad(mat, padrow, padcol):
 
 def zeros_aligned(shape, dtype, order='C', align=128):
     """Like `numpy.zeros()`, but the array will be aligned at `align` byte boundary."""
-    nbytes = numpy.prod(shape) * numpy.dtype(dtype).itemsize
-    buffer = numpy.zeros(nbytes + align, dtype=numpy.uint8)
+    nbytes = numpy.prod(shape, dtype=numpy.int64) * numpy.dtype(dtype).itemsize
+    buffer = numpy.zeros(nbytes + align, dtype=numpy.uint8)  # problematic on win64 ("maximum allowed dimension exceeded")
     start_index = -buffer.ctypes.data % align
     return buffer[start_index : start_index + nbytes].view(dtype).reshape(shape, order=order)
 
@@ -231,7 +234,7 @@ def corpus2dense(corpus, num_terms, num_docs=None, dtype=numpy.float32):
         # we know the number of documents => don't bother column_stacking
         docno, result = -1, numpy.empty((num_terms, num_docs), dtype=dtype)
         for docno, doc in enumerate(corpus):
-            numpy[:, docno] = sparse2full(doc, num_terms)
+            result[:, docno] = sparse2full(doc, num_terms)
         assert docno + 1 == num_docs
     else:
         result = numpy.column_stack(sparse2full(doc, num_terms) for doc in corpus)
@@ -272,7 +275,7 @@ class Sparse2Corpus(object):
             self.sparse = sparse.tocsr().T # make sure shape[1]=number of docs (needed in len())
 
     def __iter__(self):
-        for indprev, indnow in itertools.izip(self.sparse.indptr, self.sparse.indptr[1:]):
+        for indprev, indnow in izip(self.sparse.indptr, self.sparse.indptr[1:]):
             yield zip(self.sparse.indices[indprev:indnow], self.sparse.data[indprev:indnow])
 
     def __len__(self):
@@ -317,7 +320,7 @@ def unitvec(vec):
             return vec
 
     try:
-        first = iter(vec).next() # is there at least one element?
+        first = next(iter(vec))     # is there at least one element?
     except:
         return vec
 
@@ -336,12 +339,12 @@ def cossim(vec1, vec2):
     vec1, vec2 = dict(vec1), dict(vec2)
     if not vec1 or not vec2:
         return 0.0
-    vec1len = 1.0 * math.sqrt(sum(val * val for val in vec1.itervalues()))
-    vec2len = 1.0 * math.sqrt(sum(val * val for val in vec2.itervalues()))
+    vec1len = 1.0 * math.sqrt(sum(val * val for val in itervalues(vec1)))
+    vec2len = 1.0 * math.sqrt(sum(val * val for val in itervalues(vec2)))
     assert vec1len > 0.0 and vec2len > 0.0, "sparse documents must not contain any explicit zero entries"
     if len(vec2) < len(vec1):
         vec1, vec2 = vec2, vec1 # swap references so that we iterate over the shorter vector
-    result = sum(value * vec2.get(index, 0.0) for index, value in vec1.iteritems())
+    result = sum(value * vec2.get(index, 0.0) for index, value in iteritems(vec1))
     result /= vec1len * vec2len # rescale by vector lengths
     return result
 
@@ -395,9 +398,7 @@ class MmWriter(object):
 
     def __init__(self, fname):
         self.fname = fname
-        tmp = open(self.fname, 'w') # reset/create the target file
-        tmp.close()
-        self.fout = open(self.fname, 'rb+') # open for both reading and writing
+        self.fout = open(self.fname, 'w+') # open for both reading and writing
         self.headers_written = False
 
 
@@ -440,7 +441,7 @@ class MmWriter(object):
 
 
     @staticmethod
-    def write_corpus(fname, corpus, progress_cnt=1000, index=False, num_terms=None):
+    def write_corpus(fname, corpus, progress_cnt=1000, index=False, num_terms=None, metadata=False):
         """
         Save the vector space representation of an entire corpus to disk.
 
@@ -456,7 +457,19 @@ class MmWriter(object):
         _num_terms, num_nnz = 0, 0
         docno, poslast = -1, -1
         offsets = []
-        for docno, bow in enumerate(corpus):
+        if hasattr(corpus, 'metadata'):
+            orig_metadata = corpus.metadata
+            corpus.metadata = metadata
+            if metadata:
+                docno2metadata = {}
+        else:
+            metadata = False
+        for docno, doc in enumerate(corpus):
+            if metadata:
+                bow, data = doc
+                docno2metadata[docno] = data
+            else:
+                bow = doc
             if docno % progress_cnt == 0:
                 logger.info("PROGRESS: saving document #%i" % docno)
             if index:
@@ -468,6 +481,10 @@ class MmWriter(object):
             max_id, veclen = mw.write_vector(docno, bow)
             _num_terms = max(_num_terms, 1 + max_id)
             num_nnz += veclen
+        if metadata:
+            utils.pickle(docno2metadata, fname + '.metadata.cpickle')
+            corpus.metadata = orig_metadata
+
         num_docs = docno + 1
         num_terms = num_terms or _num_terms
 
@@ -526,9 +543,9 @@ class MmReader(object):
         """
         logger.info("initializing corpus reader from %s" % input)
         self.input, self.transposed = input, transposed
-        if isinstance(input, basestring):
+        if isinstance(input, string_types):
             input = open(input)
-        header = input.next().strip()
+        header = next(input).strip()
         if not header.lower().startswith('%%matrixmarket matrix coordinate real general'):
             raise ValueError("File %s not in Matrix Market format with coordinate real general; instead found: \n%s" %
                              (self.input, header))
@@ -568,7 +585,7 @@ class MmReader(object):
         yielded where appropriate, even if they are not explicitly stored in the
         Matrix Market file.
         """
-        if isinstance(self.input, basestring):
+        if isinstance(self.input, string_types):
             fin = open(self.input)
         else:
             fin = self.input
@@ -614,7 +631,7 @@ class MmReader(object):
         # them with a special offset, -1.
         if offset == -1:
             return []
-        if isinstance(self.input, basestring):
+        if isinstance(self.input, string_types):
             fin = open(self.input)
         else:
             fin = self.input
