@@ -51,14 +51,16 @@ with dual core Xeon 2.0GHz, 4GB RAM, ATLAS
 
 
 import logging
-import itertools
 import sys
 
 import numpy
+import scipy.linalg
 import scipy.sparse
 from scipy.sparse import sparsetools
 
 from gensim import interfaces, matutils, utils
+from gensim._six import iterkeys
+from gensim._six.moves import xrange
 
 
 logger = logging.getLogger('gensim.models.lsimodel')
@@ -189,10 +191,11 @@ class Projection(utils.SaveLoad):
             # SVD again, but over k*k^T.
             # see http://www.mail-archive.com/numpy-discussion@scipy.org/msg07224.html and
             # bug ticket http://projects.scipy.org/numpy/ticket/706
-            u_k, s_k, _ = numpy.linalg.svd(k, full_matrices=False) # TODO *ugly overkill*!! only need first self.k SVD factors... but there is no LAPACK wrapper for partial svd/eigendecomp in numpy :(
-        except numpy.linalg.LinAlgError:
+            # sdoering: replaced numpy's linalg.svd with scipy's linalg.svd:
+            u_k, s_k, _ = scipy.linalg.svd(k, full_matrices=False) # TODO *ugly overkill*!! only need first self.k SVD factors... but there is no LAPACK wrapper for partial svd/eigendecomp in numpy :( //sdoering: maybe there is one in scipy?
+        except scipy.linalg.LinAlgError:
             logger.error("SVD(A) failed; trying SVD(A * A^T)")
-            u_k, s_k, _ = numpy.linalg.svd(numpy.dot(k, k.T), full_matrices=False) # if this fails too, give up with an exception
+            u_k, s_k, _ = scipy.linalg.svd(numpy.dot(k, k.T), full_matrices=False) # if this fails too, give up with an exception
             s_k = numpy.sqrt(s_k) # go back from eigen values to singular values
 
         k = clip_spectrum(s_k**2, self.k)
@@ -268,9 +271,9 @@ class LsiModel(interfaces.TransformationABC):
         Example:
 
         >>> lsi = LsiModel(corpus, num_topics=10)
-        >>> print lsi[doc_tfidf] # project some document into LSI space
+        >>> print(lsi[doc_tfidf]) # project some document into LSI space
         >>> lsi.add_documents(corpus2) # update LSI on additional documents
-        >>> print lsi[doc_tfidf]
+        >>> print(lsi[doc_tfidf])
 
         .. [3] http://nlp.fi.muni.cz/~xrehurek/nips/rehurek_nips.pdf
 
@@ -319,7 +322,7 @@ class LsiModel(interfaces.TransformationABC):
                 self.dispatcher = dispatcher
                 self.numworkers = len(dispatcher.getworkers())
                 logger.info("using distributed version with %i workers" % self.numworkers)
-            except Exception, err:
+            except Exception as err:
                 # distributed version was specifically requested, so this is an error state
                 logger.error("failed to initialize distributed LSI (%s)" % err)
                 raise RuntimeError("failed to initialize distributed LSI (%s)" % err)
@@ -427,9 +430,24 @@ class LsiModel(interfaces.TransformationABC):
 
         if not is_corpus:
             bow = [bow]
-        vec = matutils.corpus2csc(bow, num_terms=self.num_terms)
 
+        # convert input to scipy.sparse CSC, then do "sparse * dense = dense" multiplication
+        vec = matutils.corpus2csc(bow, num_terms=self.num_terms, dtype=self.projection.u.dtype)
         topic_dist = (vec.T * self.projection.u[:, :self.num_topics]).T # (x^T * u).T = u^-1 * x
+
+        # # convert input to dense, then do dense * dense multiplication
+        # # ± same performance as above (BLAS dense * dense is better optimized than scipy.sparse), but consumes more memory
+        # vec = matutils.corpus2dense(bow, num_terms=self.num_terms, num_docs=len(bow))
+        # topic_dist = numpy.dot(self.projection.u[:, :self.num_topics].T, vec)
+
+        # # use numpy's advanced indexing to simulate sparse * dense
+        # # ± same speed again
+        # u = self.projection.u[:, :self.num_topics]
+        # topic_dist = numpy.empty((u.shape[1], len(bow)), dtype=u.dtype)
+        # for vecno, vec in enumerate(bow):
+        #     indices, data = zip(*vec) if vec else ([], [])
+        #     topic_dist[:, vecno] = numpy.dot(u.take(indices, axis=0).T, numpy.array(data, dtype=u.dtype))
+
         if scaled:
             topic_dist = (1.0 / self.projection.s[:self.num_topics]) * topic_dist # s^-1 * u^-1 * x
 
@@ -514,42 +532,32 @@ class LsiModel(interfaces.TransformationABC):
                    num_words=num_words)
 
 
-    def save(self, fname):
+    def save(self, fname, *args, **kwargs):
         """
-        Override the default `save` (which uses cPickle), because that's
-        too inefficient and cPickle has bugs. Instead, single out the large transformation
-        matrix and store that separately in binary format (that can be directly
-        mmap'ed back in `load()`), under `fname.npy`.
-        """
-        logger.info("storing %s object to %s and %s" % (self.__class__.__name__, fname, fname + '.npy'))
-        if self.projection.u is None:
-            # model not initialized: there is no projection
-            utils.pickle(self, fname)
+        Save the model to file.
 
-        # first, remove the projection from self.__dict__, so it doesn't get pickled
-        u, dispatcher = self.projection.u, self.dispatcher
-        del self.projection.u
-        self.dispatcher = None
-        try:
-            utils.pickle(self, fname) # store projection-less object
-            numpy.save(fname + '.npy', ascarray(u)) # store projection
-        finally:
-            self.projection.u, self.dispatcher = u, dispatcher
+        Large internal arrays may be stored into separate files, with `fname` as prefix.
+
+        """
+        if self.projection is not None:
+            self.projection.save(fname + '.projection', *args, **kwargs)
+        super(LsiModel, self).save(fname, *args, ignore=['projection', 'dispatcher'], **kwargs)
 
 
     @classmethod
-    def load(cls, fname):
+    def load(cls, fname, *args, **kwargs):
         """
         Load a previously saved object from file (also see `save`).
+
+        Large arrays are mmap'ed back as read-only (shared memory).
+
         """
-        logger.info("loading %s object from %s" % (cls.__name__, fname))
-        result = utils.unpickle(fname)
-        ufname = fname + '.npy'
+        kwargs['mmap'] = kwargs.get('mmap', 'r')
+        result = super(LsiModel, cls).load(fname, *args, **kwargs)
         try:
-            result.projection.u = numpy.load(ufname, mmap_mode='r') # load back as read-only
-        except:
-            logger.info("failed to load mmap'ed projection from %s" % ufname)
-        result.dispatcher = None # TODO load back incl. distributed state? will require re-initialization of worker state
+            result.projection = super(LsiModel, cls).load(fname + '.projection', *args, **kwargs)
+        except Exception as e:
+            logging.warning("failed to load projection from %s: %s" % (fname + '.state', e))
         return result
 #endclass LsiModel
 
@@ -569,7 +577,7 @@ def print_debug(id2token, u, s, topics, num_words=10, num_neg=None):
             result.setdefault(topic, []).append((udiff[topic], uvecno))
 
     logger.debug("printing %i+%i salient words" % (num_words, num_neg))
-    for topic in sorted(result.iterkeys()):
+    for topic in sorted(iterkeys(result)):
         weights = sorted(result[topic], key=lambda x: -abs(x[0]))
         _, most = weights[0]
         if u[most, topic] < 0.0: # the most significant word has a negative sign => flip sign of u[most]
@@ -695,7 +703,7 @@ def stochastic_svd(corpus, rank, num_terms, chunksize=20000, extra_dims=None,
     if scipy.sparse.issparse(corpus):
         b = qt * corpus
         logger.info("2nd phase: running dense svd on %s matrix" % str(b.shape))
-        u, s, vt = numpy.linalg.svd(b, full_matrices=False)
+        u, s, vt = scipy.linalg.svd(b, full_matrices=False)
         del b, vt
     else:
         # second phase: construct the covariance matrix X = B * B.T, where B = Q.T * A
@@ -713,7 +721,7 @@ def stochastic_svd(corpus, rank, num_terms, chunksize=20000, extra_dims=None,
 
         # now we're ready to compute decomposition of the small matrix X
         logger.info("running dense decomposition on %s covariance matrix" % str(x.shape))
-        u, s, vt = numpy.linalg.svd(x) # could use linalg.eigh, but who cares... and svd returns the factors already sorted :)
+        u, s, vt = scipy.linalg.svd(x) # could use linalg.eigh, but who cares... and svd returns the factors already sorted :)
         s = numpy.sqrt(s) # sqrt to go back from singular values of X to singular values of B = singular values of the corpus
     q = qt.T.copy()
     del qt
