@@ -6,12 +6,14 @@
 
 
 """
-Deep learning via word2vec's "hierarchical softmax skip-gram model" [1]_.
+Deep learning via word2vec's "hierarchical softmax skip-gram and CBOW models" [1]_.
 
 The training algorithm was originally ported from the C package https://code.google.com/p/word2vec/
 and extended with additional functionality.
 
-**Install Cython with `pip install cython` before to use optimized word2vec training** (70x speedup [2]_).
+For a blog tutorial on gensim word2vec, with an interactive web app trained on GoogleNews, visit http://radimrehurek.com/2014/02/word2vec-tutorial/
+
+**Install Cython with `pip install cython` to use optimized word2vec training** (70x speedup [2]_).
 
 Initialize a model with e.g.::
 
@@ -44,6 +46,12 @@ are already built-in::
 
 and so on.
 
+If you're finished training a model (=no more updates, only querying), you can do
+
+  >>> model.init_sims(replace=True)
+
+to trim unneeded model memory = use (much) less RAM.
+
 .. [1] Tomas Mikolov, Kai Chen, Greg Corrado, and Jeffrey Dean. Efficient Estimation of Word Representations in Vector Space. In Proceedings of Workshop at ICLR, 2013.
 .. [2] Optimizing word2vec in gensim, http://radimrehurek.com/2013/09/word2vec-in-python-part-two-optimizing/
 """
@@ -53,29 +61,33 @@ import sys
 import os
 import heapq
 import time
-import itertools
 import threading
-from Queue import Queue
+try:
+    from queue import Queue
+except ImportError:
+    from Queue import Queue
 
-from numpy import exp, dot, outer, random, dtype, get_include, float32 as REAL,\
-    uint32, seterr, array, uint8, vstack, argsort, fromstring, sqrt, newaxis
+from numpy import exp, dot, zeros, outer, random, dtype, get_include, float32 as REAL,\
+    uint32, seterr, array, uint8, vstack, argsort, fromstring, sqrt, newaxis, ndarray, empty
 
 logger = logging.getLogger("gensim.models.word2vec")
 
 
 from gensim import utils, matutils  # utility fnc for pickling, common scipy operations etc
+from gensim._six import iteritems, itervalues, string_types
+from gensim._six.moves import xrange
 
 
 try:
     # try to compile and use the faster cython version
     import pyximport
     pyximport.install(setup_args={"include_dirs": get_include()})
-    from word2vec_inner import train_sentence, FAST_VERSION
+    from word2vec_inner import train_sentence_sg, train_sentence_cbow, FAST_VERSION
 except:
     # failed... fall back to plain numpy (20-80x slower training than the above)
     FAST_VERSION = -1
 
-    def train_sentence(model, sentence, alpha, work=None):
+    def train_sentence_sg(model, sentence, alpha, work=None):
         """
         Update skip-gram hierarchical softmax model by training on a single sentence.
 
@@ -108,6 +120,47 @@ except:
 
         return len([word for word in sentence if word is not None])
 
+    def train_sentence_cbow(model, sentence, alpha, work=None, neu1=None):
+        """
+        Update CBOW hierarchical softmax model by training on a single sentence.
+
+        The sentence is a list of Vocab objects (or None, where the corresponding
+        word is not in the vocabulary. Called internally from `Word2Vec.train()`.
+
+        """
+
+        for pos, word in enumerate(sentence):
+            if word is None:
+                continue  # OOV word in the input sentence => skip
+            reduced_window = random.randint(model.window)  # `b` in the original word2vec code
+
+            # Combine all context words into an appropriate input
+            start = max(0, pos - model.window + reduced_window)
+            l1 = matutils.zeros_aligned((model.layer1_size), dtype=REAL)
+            count = 0
+            for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
+                if pos2 == pos or word2 is None:
+                    pass
+                else:
+                    count += 1
+                    l1 += model.syn0[word2.index]
+
+            if count > 0:
+                l1 = l1 / count
+
+            l2a = model.syn1[word.point]  # 2d matrix, codelen x layer1_size
+            fa = 1.0 / (1.0 + exp(-dot(l1, l2a.T)))  #  propagate hidden -> output
+            ga = (1 - word.code - fa) * alpha  # vector of error gradients multiplied by the learning rate
+            model.syn1[word.point] += outer(ga, l1)  # learn hidden -> output
+
+            for pos2, word2 in enumerate(sentence[start : pos + model.window + 1 - reduced_window], start):
+                if pos2 == pos or word2 is None:
+                    pass
+                else:
+                    model.syn0[word2.index] += dot(ga, l2a)
+
+        return len([word for word in sentence if word is not None])
+
 
 class Vocab(object):
     """A single vocabulary item, used internally for constructing binary trees (incl. both word leaves and inner nodes)."""
@@ -131,7 +184,7 @@ class Word2Vec(utils.SaveLoad):
     compatible with the original word2vec implementation via `save_word2vec_format()` and `load_word2vec_format()`.
 
     """
-    def __init__(self, sentences=None, size=100, alpha=0.025, window=5, min_count=5, seed=1, workers=1, min_alpha=0.0001):
+    def __init__(self, sentences=None, size=100, alpha=0.025, window=5, min_count=5, seed=1, workers=1, min_alpha=0.0001, sg=1):
         """
         Initialize the model from an iterable of `sentences`. Each sentence is a
         list of words (utf8 strings) that will be used for training.
@@ -144,6 +197,7 @@ class Word2Vec(utils.SaveLoad):
         If you don't supply `sentences`, the model is left uninitialized -- use if
         you plan to initialize it in some other way.
 
+        `sg` defines the training algorithm. By default (`sg=1`), skip-gram is used. Otherwise, `cbow` is employed.
         `size` is the dimensionality of the feature vectors.
         `window` is the maximum distance between the current and predicted word within a sentence.
         `alpha` is the initial learning rate (will linearly drop to zero as training progresses).
@@ -154,7 +208,10 @@ class Word2Vec(utils.SaveLoad):
         """
         self.vocab = {}  # mapping from a word (string) to a Vocab object
         self.index2word = []  # map from a word's matrix index (int) to word (string)
+        self.sg = int(sg)
         self.layer1_size = int(size)
+        if size % 4 != 0:
+            logger.warning("consider setting layer size to a multiple of 4 for greater performance")
         self.alpha = float(alpha)
         self.window = int(window)
         self.seed = seed
@@ -222,7 +279,7 @@ class Word2Vec(utils.SaveLoad):
 
         # assign a unique index to each word
         self.vocab, self.index2word = {}, []
-        for word, v in vocab.iteritems():
+        for word, v in iteritems(vocab):
             if v.count >= self.min_count:
                 v.index = len(self.vocab)
                 self.index2word.append(word)
@@ -249,13 +306,14 @@ class Word2Vec(utils.SaveLoad):
             raise RuntimeError("you must first build vocabulary before training the model")
 
         start, next_report = time.time(), [1.0]
-        word_count, total_words = [word_count], total_words or sum(v.count for v in self.vocab.itervalues())
+        word_count, total_words = [word_count], total_words or sum(v.count for v in itervalues(self.vocab))
         jobs = Queue(maxsize=2 * self.workers)  # buffer ahead only a limited number of jobs.. this is the reason we can't simply use ThreadPool :(
         lock = threading.Lock()  # for shared state (=number of words trained so far, log reports...)
 
         def worker_train():
             """Train the model, lifting lists of sentences from the jobs queue."""
-            work = matutils.zeros_aligned(self.layer1_size, dtype=REAL)  # each thread must have its own work memory
+            work = zeros(self.layer1_size, dtype=REAL)  # each thread must have its own work memory
+            neu1 = matutils.zeros_aligned(self.layer1_size, dtype=REAL)
 
             while True:
                 job = jobs.get()
@@ -264,7 +322,10 @@ class Word2Vec(utils.SaveLoad):
                 # update the learning rate before every job
                 alpha = max(self.min_alpha, self.alpha * (1 - 1.0 * word_count[0] / total_words))
                 # how many words did we train on? out-of-vocabulary (unknown) words do not count
-                job_words = sum(train_sentence(self, sentence, alpha, work) for sentence in job)
+                if self.sg:
+                    job_words = sum(train_sentence_sg(self, sentence, alpha, work) for sentence in job)
+                else:
+                    job_words = sum(train_sentence_cbow(self, sentence, alpha, work, neu1) for sentence in job)
                 with lock:
                     word_count[0] += job_words
                     elapsed = time.time() - start
@@ -299,25 +360,33 @@ class Word2Vec(utils.SaveLoad):
 
     def reset_weights(self):
         """Reset all projection weights to an initial (untrained) state, but keep the existing vocabulary."""
+        logger.info("resetting layer weights")
         random.seed(self.seed)
-        self.syn0 = matutils.zeros_aligned((len(self.vocab), self.layer1_size), dtype=REAL)
-        self.syn1 = matutils.zeros_aligned((len(self.vocab), self.layer1_size), dtype=REAL)
-        self.syn0 += (random.rand(len(self.vocab), self.layer1_size) - 0.5) / self.layer1_size
+        self.syn0 = empty((len(self.vocab), self.layer1_size), dtype=REAL)
+        # randomize weights vector by vector, rather than materializing a huge random matrix in RAM at once
+        for i in xrange(len(self.vocab)):
+            self.syn0[i] = (random.rand(self.layer1_size) - 0.5) / self.layer1_size
+        self.syn1 = zeros((len(self.vocab), self.layer1_size), dtype=REAL)
         self.syn0norm = None
 
 
-    def save_word2vec_format(self, fname, binary=False):
+    def save_word2vec_format(self, fname, fvocab=None, binary=False):
         """
         Store the input-hidden weight matrix in the same format used by the original
         C word2vec-tool, for compatibility.
 
         """
+        if fvocab is not None:
+            logger.info("Storing vocabulary in %s" % (fvocab))
+            with utils.smart_open(fvocab, 'wb') as vout:
+                for word, vocab in sorted(iteritems(self.vocab), key=lambda item: -item[1].count):
+                    vout.write("%s %s\n" % (word, vocab.count))
         logger.info("storing %sx%s projection weights into %s" % (len(self.vocab), self.layer1_size, fname))
         assert (len(self.vocab), self.layer1_size) == self.syn0.shape
-        with open(fname, 'wb') as fout:
+        with utils.smart_open(fname, 'wb') as fout:
             fout.write("%s %s\n" % self.syn0.shape)
             # store in sorted order: most frequent words at the top
-            for word, vocab in sorted(self.vocab.iteritems(), key=lambda item: -item[1].count):
+            for word, vocab in sorted(iteritems(self.vocab), key=lambda item: -item[1].count):
                 word = utils.to_utf8(word)  # always store in utf8
                 row = self.syn0[vocab.index]
                 if binary:
@@ -327,21 +396,32 @@ class Word2Vec(utils.SaveLoad):
 
 
     @classmethod
-    def load_word2vec_format(cls, fname, binary=False):
+    def load_word2vec_format(cls, fname, fvocab=None, binary=False, norm_only=True):
         """
         Load the input-hidden weight matrix from the original C word2vec-tool format.
 
-        Note that the information loaded is incomplete (the binary tree is missing),
+        Note that the information stored in the file is incomplete (the binary tree is missing),
         so while you can query for word similarity etc., you cannot continue training
         with a model loaded this way.
 
+        `binary` is a boolean indicating whether the data is in binary word2vec format
+        `norm_only` is a boolean indicating whether to only store normalised word2vec vectors in memory
         """
+        counts = None
+        if fvocab is not None:
+            logger.info("loading word counts from %s" % (fvocab))
+            counts = {}
+            with utils.smart_open(fvocab) as fin:
+                for line in fin:
+                    word, count = line.strip().split()
+                    counts[word] = int(count)
+
         logger.info("loading projection weights from %s" % (fname))
-        with open(fname) as fin:
+        with utils.smart_open(fname) as fin:
             header = fin.readline()
             vocab_size, layer1_size = map(int, header.split())  # throws for invalid file format
             result = Word2Vec(size=layer1_size)
-            result.syn0 = matutils.zeros_aligned((vocab_size, layer1_size), dtype=REAL)
+            result.syn0 = zeros((vocab_size, layer1_size), dtype=REAL)
             if binary:
                 binary_len = dtype(REAL).itemsize * layer1_size
                 for line_no in xrange(vocab_size):
@@ -354,19 +434,32 @@ class Word2Vec(utils.SaveLoad):
                             break
                         if ch != '\n':  # ignore newlines in front of words (some binary files have newline, some not)
                             word.append(ch)
-                    result.vocab[word] = Vocab(index=line_no, count=vocab_size - line_no)
+                    if counts is None:
+                        result.vocab[word] = Vocab(index=line_no, count=vocab_size - line_no)
+                    elif counts.has_key(word):
+                        result.vocab[word] = Vocab(index=line_no, count=counts[word])
+                    else:
+                        logger.warning("vocabulary file is incomplete")
+                        result.vocab[word] = Vocab(index=line_no, count=None)
                     result.index2word.append(word)
                     result.syn0[line_no] = fromstring(fin.read(binary_len), dtype=REAL)
             else:
                 for line_no, line in enumerate(fin):
                     parts = line.split()
-                    assert len(parts) == layer1_size + 1
+                    if len(parts) != layer1_size + 1:
+                        raise ValueError("invalid vector on line %s (is this really the text format?)" % (line_no))
                     word, weights = parts[0], map(REAL, parts[1:])
-                    result.vocab[word] = Vocab(index=line_no, count=vocab_size - line_no)
+                    if counts is None:
+                        result.vocab[word] = Vocab(index=line_no, count=vocab_size - line_no)
+                    elif counts.has_key(word):
+                        result.vocab[word] = Vocab(index=line_no, count=counts[word])
+                    else:
+                        logger.warning("vocabulary file is incomplete")
+                        result.vocab[word] = Vocab(index=line_no, count=None)
                     result.index2word.append(word)
                     result.syn0[line_no] = weights
         logger.info("loaded %s matrix from %s" % (result.syn0.shape, fname))
-        result.init_sims()
+        result.init_sims(norm_only)
         return result
 
 
@@ -387,19 +480,23 @@ class Word2Vec(utils.SaveLoad):
         """
         self.init_sims()
 
-        if isinstance(positive, basestring) and not negative:
+        if isinstance(positive, string_types) and not negative:
             # allow calls like most_similar('dog'), as a shorthand for most_similar(['dog'])
             positive = [positive]
 
         # add weights for each word, if not already present; default to 1.0 for positive and -1.0 for negative words
-        positive = [(word, 1.0) if isinstance(word, basestring) else word for word in positive]
-        negative = [(word, -1.0) if isinstance(word, basestring) else word for word in negative]
+        positive = [(word, 1.0) if isinstance(word, string_types + (ndarray,))
+                                else word for word in positive]
+        negative = [(word, -1.0) if isinstance(word, string_types + (ndarray,))
+                                 else word for word in negative]
 
         # compute the weighted average of all words
         all_words, mean = set(), []
         for word, weight in positive + negative:
-            if word in self.vocab:
-                mean.append(weight * matutils.unitvec(self.syn0[self.vocab[word].index]))
+            if isinstance(word, ndarray):
+                mean.append(weight * word)
+            elif word in self.vocab:
+                mean.append(weight * self.syn0norm[self.vocab[word].index])
                 all_words.add(self.vocab[word].index)
             else:
                 raise KeyError("word '%s' not in vocabulary" % word)
@@ -412,7 +509,7 @@ class Word2Vec(utils.SaveLoad):
             return dists
         best = argsort(dists)[::-1][:topn + len(all_words)]
         # ignore (don't return) words from the input
-        result = [(self.index2word[sim], dists[sim]) for sim in best if sim not in all_words]
+        result = [(self.index2word[sim], float(dists[sim])) for sim in best if sim not in all_words]
         return result[:topn]
 
 
@@ -426,11 +523,13 @@ class Word2Vec(utils.SaveLoad):
           'cereal'
 
         """
+        self.init_sims()
+
         words = [word for word in words if word in self.vocab]  # filter out OOV words
         logger.debug("using words %s" % words)
         if not words:
             raise ValueError("cannot select a word from an empty list")
-        vectors = vstack(matutils.unitvec(self.syn0[self.vocab[word].index]) for word in words).astype(REAL)
+        vectors = vstack(self.syn0norm[self.vocab[word].index] for word in words).astype(REAL)
         mean = matutils.unitvec(vectors.mean(axis=0)).astype(REAL)
         dists = dot(vectors, mean)
         return sorted(zip(dists, words))[0][1]
@@ -469,10 +568,27 @@ class Word2Vec(utils.SaveLoad):
         return dot(matutils.unitvec(self[w1]), matutils.unitvec(self[w2]))
 
 
-    def init_sims(self):
-        if getattr(self, 'syn0norm', None) is None:
+    def init_sims(self, replace=False):
+        """
+        Precompute L2-normalized vectors.
+
+        If `replace` is set, forget the original vectors and only keep the normalized
+        ones = saves lots of memory!
+
+        Note that you **cannot continue training** after doing a replace. The model becomes
+        effectively read-only = you can call `most_similar`, `similarity` etc., but not `train`.
+
+        """
+        if getattr(self, 'syn0norm', None) is None or replace:
             logger.info("precomputing L2-norms of word weight vectors")
-            self.syn0norm = (self.syn0 / sqrt((self.syn0 ** 2).sum(-1))[..., newaxis]).astype(REAL)
+            if replace:
+                for i in range(self.syn0.shape[0]):
+                    self.syn0[i, :] /= sqrt((self.syn0[i, :] ** 2).sum(-1))
+                self.syn0norm = self.syn0
+                if hasattr(self, 'syn1'):
+                    del self.syn1
+            else:
+                self.syn0norm = (self.syn0 / sqrt((self.syn0 ** 2).sum(-1))[..., newaxis]).astype(REAL)
 
 
     def accuracy(self, questions, restrict_vocab=30000):
@@ -490,8 +606,9 @@ class Word2Vec(utils.SaveLoad):
         This method corresponds to the `compute-accuracy` script of the original C word2vec.
 
         """
-        ok_vocab = dict(sorted(self.vocab.iteritems(), key=lambda item: -item[1].count)[:restrict_vocab])
-        ok_index = set(v.index for v in ok_vocab.itervalues())
+        ok_vocab = dict(sorted(iteritems(self.vocab),
+                               key=lambda item: -item[1].count)[:restrict_vocab])
+        ok_index = set(v.index for v in itervalues(ok_vocab))
 
         def log_accuracy(section):
             correct, incorrect = section['correct'], section['incorrect']
@@ -545,6 +662,10 @@ class Word2Vec(utils.SaveLoad):
         return "Word2Vec(vocab=%s, size=%s, alpha=%s)" % (len(self.index2word), self.layer1_size, self.alpha)
 
 
+    def save(self, *args, **kwargs):
+        kwargs['ignore'] = kwargs.get('ignore', ['syn0norm']) # don't bother storing the cached normalized vectors
+        super(Word2Vec, self).save(*args, **kwargs)
+
 
 class BrownCorpus(object):
     """Iterate over sentences from the Brown corpus (part of NLTK data)."""
@@ -576,7 +697,7 @@ class Text8Corpus(object):
         # the entire corpus is one gigantic line -- there are no sentence marks at all
         # so just split the sequence of tokens arbitrarily: 1 sentence = 1000 tokens
         sentence, rest, max_sentence_length = [], '', 1000
-        with open(self.fname) as fin:
+        with utils.smart_open(self.fname) as fin:
             while True:
                 text = rest + fin.read(8192)  # avoid loading the entire file (=1 line) into RAM
                 if text == rest:  # EOF
@@ -618,8 +739,9 @@ class LineSentence(object):
                 yield line.split()
         except AttributeError:
             # If it didn't work like a file, use it as a string filename
-            for line in open(self.source):
-                yield line.split()
+            with utils.smart_open(self.source) as fin:
+                for line in fin:
+                    yield line.split()
 
 
 
@@ -632,7 +754,7 @@ if __name__ == "__main__":
     # check and process cmdline input
     program = os.path.basename(sys.argv[0])
     if len(sys.argv) < 2:
-        print globals()['__doc__'] % locals()
+        print(globals()['__doc__'] % locals())
         sys.exit(1)
     infile = sys.argv[1]
     from gensim.models.word2vec import Word2Vec  # avoid referencing __main__ in pickle
