@@ -16,6 +16,7 @@ import os.path
 import tempfile
 
 import numpy
+import scipy.linalg
 
 from gensim.corpora import mmcorpus, Dictionary
 from gensim.models import lsimodel, ldamodel, tfidfmodel, rpmodel, logentropy_model
@@ -55,7 +56,7 @@ class TestLsiModel(unittest.TestCase):
         model = lsimodel.LsiModel(self.corpus, num_topics=2)
 
         # make sure the decomposition is enough accurate
-        u, s, vt = numpy.linalg.svd(matutils.corpus2dense(self.corpus, self.corpus.num_terms), full_matrices=False)
+        u, s, vt = scipy.linalg.svd(matutils.corpus2dense(self.corpus, self.corpus.num_terms), full_matrices=False)
         self.assertTrue(numpy.allclose(s[:2], model.projection.s)) # singular values must match
 
         # transform one document
@@ -128,6 +129,27 @@ class TestLsiModel(unittest.TestCase):
         self.assertTrue(numpy.allclose(model.projection.s, model2.projection.s))
         tstvec = []
         self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+    def testLargeMmap(self):
+        model = lsimodel.LsiModel(self.corpus, num_topics=2)
+
+        # test storing the internal arrays into separate files
+        model.save(testfile(), sep_limit=0)
+
+        model2 = lsimodel.LsiModel.load(testfile())
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.projection.u, model2.projection.u))
+        self.assertTrue(numpy.allclose(model.projection.s, model2.projection.s))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+        # now load the external arrays via mmap
+        model2 = lsimodel.LsiModel.load(testfile(), mmap='r')
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.projection.u, model2.projection.u))
+        self.assertTrue(numpy.allclose(model.projection.s, model2.projection.s))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
 #endclass TestLsiModel
 
 
@@ -169,7 +191,7 @@ class TestLdaModel(unittest.TestCase):
         # sometimes, LDA training gets stuck at a local minimum
         # in that case try re-training the model from scratch, hoping for a
         # better random initialization
-        for i in xrange(5): # restart at most 5 times
+        for i in range(5): # restart at most 5 times
             # create the transformation model
             model = ldamodel.LdaModel(id2word=dictionary, num_topics=2, passes=100)
             model.update(corpus)
@@ -196,6 +218,25 @@ class TestLdaModel(unittest.TestCase):
         self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
         tstvec = []
         self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+    def testLargeMmap(self):
+        model = ldamodel.LdaModel(self.corpus, num_topics=2)
+
+        # simulate storing large arrays separately
+        model.save(testfile(), sep_limit=0)
+
+        model2 = ldamodel.LdaModel.load(testfile())
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
+
+        # test loading the large model arrays with mmap
+        model2 = ldamodel.LdaModel.load(testfile(), mmap='r')
+        self.assertEqual(model.num_topics, model2.num_topics)
+        self.assertTrue(numpy.allclose(model.expElogbeta, model2.expElogbeta))
+        tstvec = []
+        self.assertTrue(numpy.allclose(model[tstvec], model2[tstvec])) # try projecting an empty vector
 #endclass TestLdaModel
 
 
@@ -272,5 +313,5 @@ class TestLogEntropyModel(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    logging.root.setLevel(logging.DEBUG)
+    logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.DEBUG)
     unittest.main()
